﻿using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using Microsoft.Win32;
using ScintillaNet;

namespace RWTextEdit
{
    public partial class Main : Form
    {
        string RWDirectory = "C:\\Program Files (x86)\\Steam\\steamapps\\common\\railworks";
        string RWRegistryPath = "HKEY_CURRENT_USER\\Software\\Ivimey.Com\\RWTextEdit";
        string RWName = "RWTextEdit";

        OpenFileDialog openFile;
        SaveFileDialog saveFile;
        string tempFilename;        // unencoded temp file
        string origFilename;        // name of encoded file
        string windowTitle;
        string serzPath;
        string[] _args;

        private int NumMRUItems { get; set; }
        private MRUList<string> mruList;

        INativeScintilla iscintilla;
        Options optsDlg;

        public Main()
        {
            InitializeComponent();

            // try to work out where the railworks folder is.
            string steamPath = (string)Registry.GetValue(
                        "HKEY_CURRENT_USER\\Software\\Valve\\Steam", "SteamPath", null);
            if (String.IsNullOrEmpty(steamPath))
                RWDirectory = "C:\\Program Files\\Train Simulator";
            else
            {
                string rwPath = (string)Registry.GetValue(
                        "HKEY_LOCAL_MACHINE\\Software\\Wow6432Node\\RailSimulator.com\\RailWorks",
                        "Install_Path", null);
                if (String.IsNullOrEmpty(rwPath))
                {
                    rwPath = (string)Registry.GetValue(
                        "HKEY_LOCAL_MACHINE\\Software\\RailSimulator.com\\RailWorks",
                        "Install_Path", null);
                }
                RWDirectory = rwPath;
            }

            this.UseWaitCursor = false;
            optsDlg = new Options();
            serzPath = Path.Combine(RWDirectory, "serz.exe");

            int tabOption = (int)Registry.GetValue(RWRegistryPath, "Tab", 4);
            RWDirectory = (string)Registry.GetValue(RWRegistryPath, "RWDir", RWDirectory);
            serzPath = (string)Registry.GetValue(RWRegistryPath, "Serz", serzPath);
            string fontFace = (string)Registry.GetValue(RWRegistryPath, "FontFace", scintilla.Font.Name);
            string fontSize = (string)Registry.GetValue(RWRegistryPath, "FontSize", scintilla.Font.Size.ToString());

            float fontSz;
            float.TryParse(fontSize, out fontSz);
            optsDlg.EditorFont = new Font(fontFace, fontSz);

            optsDlg.RailworksPathTextbox.Text = RWDirectory;
            optsDlg.SerzPathTextbox.Text = serzPath;
            optsDlg.tabStopExpansion.Value = tabOption;

            // get the MRU list working.
            MRUList<string>.MRUEqualityComparer eq = MRUIsEqual;
            mruList = MRUList<string>.LoadFromRegistry(RWRegistryPath + "\\MRU", eq);
            mruList.OrderChanged += new MRUList<string>.OrderChangedEventHandler(mruList_OrderChanged);
            UpdateRecentMenu();

            // open file dialog
            openFile = new OpenFileDialog();
            openFile.Filter = "RW Serz Files (*.bin,*.GeoPcDx,*.TgPcDx,)|*.bin;*.GeoPcDx;*.TgPcDx|All Files (*.*)|*.*";
            openFile.Multiselect = false;
            openFile.AutoUpgradeEnabled = true;
            openFile.CheckFileExists = true;
            openFile.Title = "Open RW Serz File";
            openFile.DefaultExt = ".bin";

            // save file dialog
            saveFile = new SaveFileDialog();
            saveFile.Filter = "RW Serz Files (*.bin,*.GeoPcDx,*.TgPcDx)|*.bin;*.GeoPcDx;*.TgPcDx|All Files (*.*)|*.*";
            saveFile.AddExtension = true;
            saveFile.CheckPathExists = true;
            saveFile.DefaultExt = ".bin";
            saveFile.Title = "Save RW Serz File";

            tempFilename = null;
            scintilla.Text = "";
            //scintilla.UseFont = true;
            //scintilla.Font = optsDlg.EditorFont;
            //scintilla.Styles.Default.Font = optsDlg.EditorFont;
            scintilla.Modified = false;

            scintilla.AcceptsTab = true;
            scintilla.AcceptsReturn = true;
            scintilla.Margins.Margin0.Width = 0;
            scintilla.Margins.Margin1.Width = 0;
            scintilla.Margins.Margin2.Width = 20;
            scintilla.Margins.Margin3.Width = 0;
            scintilla.Margins.Margin4.Width = 0;
            scintilla.Folding.IsEnabled = true;

            if (whitespaceToolStripMenuItem.Checked)
                scintilla.Whitespace.Mode = WhitespaceMode.VisibleAlways;
            else
                scintilla.Whitespace.Mode = WhitespaceMode.Invisible;
            if (wordWrapToolStripMenuItem.Checked)
                scintilla.LineWrap.Mode = WrapMode.Word;
            else
                scintilla.LineWrap.Mode = WrapMode.None;

            AllowDrop = true;
            DragEnter += new DragEventHandler(Main_DragEnter);
            DragDrop += new DragEventHandler(Main_DragDrop);

            menuStrip.AllowDrop = true;
            menuStrip.DragEnter += new DragEventHandler(Main_DragEnter);
            menuStrip.DragDrop += new DragEventHandler(Main_DragDrop);

            scintilla.ModifiedChanged += new EventHandler(scintilla_ModifiedChanged);
            iscintilla = scintilla.NativeInterface;
            iscintilla.SetTabWidth((int)optsDlg.tabStopExpansion.Value);
            this.FormClosing += new FormClosingEventHandler(Main_FormClosing);
        }

        void mruList_OrderChanged(object sender, EventArgs e)
        {
            UpdateRecentMenu();
        }

        void UpdateRecentMenu()
        {
            ToolStripItem[] items = new ToolStripItem[mruList.Count];
            int i = 0;
            openRecentToolStripMenuItem.DropDownItems.Clear();
            foreach (string file in mruList)
            {
                string filename = Path.GetFileName(file);
                ToolStripButton btn = new ToolStripButton(filename);
                btn.ToolTipText = file;
                btn.Size = new Size(150, 22);
                btn.Click += new EventHandler(recentFile_Click);
                items[i++] = btn;
            }
            openRecentToolStripMenuItem.DropDownItems.AddRange(items);
        }

        void recentFile_Click(object sender, EventArgs e)
        {
            ToolStripButton btn = (ToolStripButton)sender;
            string file = btn.ToolTipText;
            DoOpen(file);
        }

        bool MRUIsEqual(string s1, string n1)
        {
            return s1 == n1;
        }

        bool IsModified
        {
            get { return scintilla.Modified; }
        }

        private void SetAllFontStyles(Font theFont)
        {
            for (int i = 0; i < 32; i++)
            {
                Style s = scintilla.Styles[i];
                s.Font = theFont;
            }
        }

        public Main(string[] args)
            : this()
        {
            // Store the command line args
            this._args = args;
        }

        private void Main_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
                if (files.Length == 1)
                    e.Effect = DragDropEffects.All;
                else
                    e.Effect = DragDropEffects.None;
            }
        }

        private void Main_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
                if (files.Length == 1)
                {
                    DoOpen(files[0]);
                }
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CleanupTemp();
            Close();
        }

        private void scintilla_ModifiedChanged(object sender, EventArgs e)
        {
            SetDocTitle();
        }

        private void Main_FormClosing(object sender, FormClosingEventArgs e)
        {
            mruList.SaveToRegistry(RWRegistryPath + "\\MRU");
            CleanupTemp();
        }

        private void Main_Load(object sender, EventArgs e)
        {
            if (_args != null && _args.Length != 0)
            {
                // Open the document specified on the command line
                FileInfo fi = new FileInfo(_args[0]);
                if (fi.Exists)
                {
                    DoOpen(fi.FullName);
                }
            }
            SetDocTitle();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (IsModified)
            {
                if (MessageBox.Show("The file has been modified.\nDo you wish to save it?",
                        "New Document", MessageBoxButtons.OKCancel) != DialogResult.OK)
                {
                    DoSave();
                }
            }
            CleanupTemp();
            origFilename = null;
            iscintilla.ClearAll();
            scintilla.Modified = false;

            SetDocTitle();
        }

        private void openButton_Click(object sender, EventArgs e)
        {
            DoOpen();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DoOpen();
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DoSaveAs();
        }

        private void saveButton_Click(object sender, EventArgs e)
        {
            DoSave();
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DoSave();
        }

        private void DoOpen()
        {
            if (IsModified)
            {
                if (MessageBox.Show("The file has been modified.\nDo you wish to save it?",
                        "Open Document", MessageBoxButtons.OKCancel) != DialogResult.OK)
                {
                    DoSave();
                }
            }
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                DoOpen(openFile.FileName);
            }
        }

        private void DoOpen(string filenam)
        {
            this.UseWaitCursor = true;

            openSerzFile(filenam);
            if (mruList.Find(filenam) == null)
                mruList.Add(filenam);

            SetAllFontStyles(optsDlg.EditorFont);
            SetDocTitle();
            this.UseWaitCursor = false;
        }

        private void DoSave()
        {
            // if there is an original filename set, the file is named so don't prompt
            if (!String.IsNullOrEmpty(origFilename))
            {
                this.UseWaitCursor = true;
                saveSerzFile(tempFilename, origFilename);
                SetDocTitle();
                this.UseWaitCursor = false;
            }
            // else we must ask for a filename.
            else
            {
                DoSaveAs();
            }
        }

        private void DoSaveAs()
        {
            if (saveFile.ShowDialog() == DialogResult.OK)
            {
                CleanupTemp();
                origFilename = saveFile.FileName;
                tempFilename = GetTempFileName(origFilename);

                DoSave();
            }
        }

        void CleanupTemp()
        {
            if (!String.IsNullOrEmpty(tempFilename))
            {
                System.IO.File.Delete(tempFilename);
                tempFilename = null;
            }
        }

        private void cutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            scintilla.Clipboard.Cut();
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            scintilla.Clipboard.Copy();
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            scintilla.Clipboard.Paste();
        }

        private void clearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            iscintilla.DeleteBack();
        }

        private void undoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            scintilla.UndoRedo.Undo();
        }

        private void redoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            scintilla.UndoRedo.Redo();
        }

        private void cutButton1_Click(object sender, EventArgs e)
        {
            iscintilla.Cut();
        }

        private void copyButton1_Click(object sender, EventArgs e)
        {
            iscintilla.Copy();
        }

        private void pasteButton1_Click(object sender, EventArgs e)
        {
            iscintilla.Paste();
        }

        private void undoButton_Click(object sender, EventArgs e)
        {
            scintilla.UndoRedo.Undo();
        }

        private void redoButton_Click(object sender, EventArgs e)
        {
            scintilla.UndoRedo.Redo();
        }

        private void whitespaceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            whitespaceToolStripMenuItem.Checked = !whitespaceToolStripMenuItem.Checked;
            if (whitespaceToolStripMenuItem.Checked)
                scintilla.Whitespace.Mode = WhitespaceMode.VisibleAlways;
            else
                scintilla.Whitespace.Mode = WhitespaceMode.Invisible;
        }

        private void wordWrapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            wordWrapToolStripMenuItem.Checked = !wordWrapToolStripMenuItem.Checked;
            if (wordWrapToolStripMenuItem.Checked)
                scintilla.LineWrap.Mode = WrapMode.Word;
            else
                scintilla.LineWrap.Mode = WrapMode.None;
        }

        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            optsDlg.ShowDialog(this);

            // Store locally
            iscintilla.SetTabWidth((int)optsDlg.tabStopExpansion.Value);
            RWDirectory = optsDlg.RailworksPathTextbox.Text;
            serzPath = optsDlg.SerzPathTextbox.Text;
            //scintilla.UseFont = true;
            SetAllFontStyles(optsDlg.EditorFont);
            
            // Save to the registry
            Microsoft.Win32.Registry.SetValue(
                        "HKEY_CURRENT_USER\\Software\\Ivimey.Com\\RWTextEdit", "Tab", (int)optsDlg.tabStopExpansion.Value, Microsoft.Win32.RegistryValueKind.DWord);
            Microsoft.Win32.Registry.SetValue(
                        "HKEY_CURRENT_USER\\Software\\Ivimey.Com\\RWTextEdit", "RWDir", RWDirectory, Microsoft.Win32.RegistryValueKind.ExpandString);
            Microsoft.Win32.Registry.SetValue(
                        "HKEY_CURRENT_USER\\Software\\Ivimey.Com\\RWTextEdit", "Serz", serzPath, Microsoft.Win32.RegistryValueKind.ExpandString);
            Microsoft.Win32.Registry.SetValue(
                        "HKEY_CURRENT_USER\\Software\\Ivimey.Com\\RWTextEdit", "FontFace", optsDlg.EditorFont.Name, Microsoft.Win32.RegistryValueKind.String);
            Microsoft.Win32.Registry.SetValue(
                        "HKEY_CURRENT_USER\\Software\\Ivimey.Com\\RWTextEdit", "FontSize", optsDlg.EditorFont.Size.ToString(), Microsoft.Win32.RegistryValueKind.String);
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox abt = new AboutBox();
            abt.ShowDialog();
        }

        private void findButton_Click(object sender, EventArgs e)
        {
            findToolStripMenuItem_Click(sender, e);
        }

        private void findToolStripMenuItem_Click(object sender, EventArgs e)
        {
            scintilla.FindReplace.ShowFind();
        }

        private void replaceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            scintilla.FindReplace.ShowReplace();
        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            scintilla.Printing.Print();
        }

        private void goToToolStripMenuItem_Click(object sender, EventArgs e)
        {
            scintilla.GoTo.ShowGoToDialog();
        }

        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            scintilla.Selection.SelectAll();
        }





        private void saveSerzFile(string tempFilename, string saveFilename)
        {
            // fetch the text from the control
            string docText;
            iscintilla.GetText(iscintilla.GetTextLength(), out docText);

            // save (back) to the temporary file
            using (FileStream fs = new FileStream(tempFilename, FileMode.Create))
            {
                StreamWriter sw = new StreamWriter(fs);
                sw.Write(docText);
                sw.Close();
            }

            // reencode back into serz format
            EncodeSerz(tempFilename, saveFilename);
            scintilla.Modified = false;
        }

        private void openSerzFile(string openFilename)
        {
            tempFilename = DecodeSerz(openFilename);
            origFilename = openFilename;

            try
            {
                scintilla.Text = File.ReadAllText(tempFilename);
            }
            catch (Exception)
            {
                tempFilename = null;
                origFilename = null;
                return;
            }

            scintilla.UndoRedo.EmptyUndoBuffer();
            scintilla.Modified = false;
            scintilla.ConfigurationManager.Language = "xml";
            scintilla.Indentation.SmartIndentType = SmartIndent.None;
        }


        private void SetDocTitle()
        {
            windowTitle = RWName;
            if (!String.IsNullOrEmpty(tempFilename) && !String.IsNullOrEmpty(origFilename))
            {
                string titleFilename = origFilename;
                string marker = "\\steamapps\\common";
                int i = titleFilename.IndexOf(marker, StringComparison.CurrentCultureIgnoreCase);
                if (i >= 0)
                {
                    titleFilename = origFilename.Remove(0, i + marker.Length);
                    titleFilename = "%STEAM%" + titleFilename;
                }
                windowTitle += " - " + titleFilename + " (" + Path.GetFileName(tempFilename) + ")";
            }
            if (IsModified)
                this.Text = windowTitle + " *";
            else
                this.Text = windowTitle;
        }


        /// <summary>
        /// Return a name suitable for use as a temporary file name. This algo. isn't
        /// safe, as there could be a race between two processes asking for the name
        /// and using it, but the race depends on human interaction speeds and so is
        /// very unlikely indeed.
        /// </summary>
        /// <param name="origPath">The name of the original file - /path/to/xxx.bin, etc</param>
        /// <returns>Path to temp file in user's temp directory : /temp/path/to/xxx.xml</returns>
        private string GetTempFileName(string origPath)
        {
            string tempDir = System.IO.Path.GetTempPath();
            string fName = Path.GetFileNameWithoutExtension(origPath);
            fName = Path.Combine(tempDir, fName);
            
            // check the file is unique
            string fUnique = fName + ".xml";
            if (!File.Exists(fUnique))
                return fUnique;

            // if not, try making numbered versions
            for (int n = 1; n < 99; n++)
            {
                fUnique = fName + "_" + n.ToString() + ".xml";
                if (!File.Exists(fUnique))
                    return fUnique;
            }

            // give up!
            return fUnique;
        }

        private void EncodeSerz(string tempInpath, string outPath)
        {
            string args = "\"" + tempInpath + "\" /:\"" + outPath + "\"";
            Process serz;
            try
            {
                ProcessStartInfo si = new ProcessStartInfo(serzPath, args);
                si.CreateNoWindow = true;
                serz = Process.Start(si);
                serz.WaitForExit(1000 * 20 /* ms */);
                if (!serz.HasExited)
                    serz.Kill();
                serz.Close();
            }
            catch(Win32Exception e)
			{
                if (e.NativeErrorCode == SystemErrorCodes.ERROR_FILE_NOT_FOUND)
				{
					MessageBox.Show(e.Message + ". Check the path.",
                        "Executing Serz.exe", MessageBoxButtons.OK);
				}

                else if (e.NativeErrorCode == SystemErrorCodes.ERROR_ACCESS_DENIED)
				{
					// Note that if your word processor might generate exceptions
					// such as this, which are handled first.
                    MessageBox.Show(e.Message + 
						". You do not have permission to run this file.",
                        "Executing Serz.exe", MessageBoxButtons.OK);
				}
			}
        }

        private string DecodeSerz(string inpath)
        {
            string tempFile = GetTempFileName(inpath);
            Process serz;

            string args = "\"" + inpath + "\" /:\"" + tempFile + "\"";
            try
            {
                ProcessStartInfo si = new ProcessStartInfo(serzPath, args);
                si.CreateNoWindow = true;
                serz = Process.Start(si);
                serz.WaitForExit(1000 * 20 /* ms */);
                if (!serz.HasExited)
                    serz.Kill();
                serz.Close();
            }
            catch (Win32Exception e)
            {
                if (e.NativeErrorCode == SystemErrorCodes.ERROR_FILE_NOT_FOUND)
                {
                    MessageBox.Show(e.Message + ". Check the path.",
                        "Executing Serz.exe", MessageBoxButtons.OK);
                }

                else if (e.NativeErrorCode == SystemErrorCodes.ERROR_ACCESS_DENIED)
                {
                    // Note that if your word processor might generate exceptions
                    // such as this, which are handled first.
                    MessageBox.Show(e.Message +
                        ". You do not have permission to run this file.",
                        "Executing Serz.exe", MessageBoxButtons.OK);
                }
            }
            return tempFile;
        }


    }
}
